/*
 * Payload initialisation functions.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdint.h>



extern uint8_t _data_original, _data, _bss;

/* These are defined using SIZEOF and their table entries contain actual values.
   The "address" provided by each entry is therefore the value! */

extern uint32_t _data_size, _bss_size;

/* Relocate the .data section's contents to initialise mutable structures. */

void relocate_data(void)
{
    uint8_t *original = &_data_original, *current = &_data,
            *limit = original + (uint32_t) &_data_size;

    /* Copy data from program memory to the relocated data area in RAM. */

    while (original < limit)
        *current++ = *original++;
}

/* Clear the .bss section. */

void init_bss(void)
{
    uint8_t *current = &_bss, *limit = current + (uint32_t) &_bss_size;

    while (current < limit)
        *current++ = 0;
}
